#!/usr/bin/env python

import numpy as np
import matplotlib.animation as animation
from matplotlib import pyplot as plt
from matplotlib.backends.qt_compat import QtWidgets
from matplotlib.widgets import Slider
from scipy.special import j0, y0

from logslider import LogSlider0

π = np.pi

r = np.linspace(0., 16., 321)
θ = np.linspace(-0.5 * π, 0.5 * π, 361)
rr, θθ = np.meshgrid(r, θ)

Δt = 0.1
δ = np.exp(-2j * π * Δt)

x = np.linspace(-0.5 * π, 0.5 * π, 1441)

H0 = lambda r: j0(2 * π * r) + 1j * y0(2 * π * r)


def wave(d):
    yy = rr * np.sin(θθ)
    xx = rr * np.cos(θθ)
    rr1 = np.sqrt(xx**2 + (yy - 0.5 * d)**2)
    rr2 = np.sqrt(xx**2 + (yy + 0.5 * d)**2)
    res = H0(rr1) + H0(rr2)
    res *= 3.
    return res


def farfield(d):
    Δφ = 2 * π * d * np.sin(x)
    return 4. * np.cos(0.5 * Δφ)**2


fig = plt.figure(figsize=(16, 9))
manager = fig.canvas.manager
manager.set_window_title("Dwie szczeliny")
manager.window.findChild(QtWidgets.QToolBar).setVisible(False)
manager.full_screen_toggle()

axw = fig.add_subplot(121, polar='True')
axw.axis('off')

psi = wave(0.5)

img = axw.pcolormesh(θθ, rr, psi.real, cmap='bwr', shading='auto', vmin=-2, vmax=2)
axw.grid(False)

ro = np.array([0.35, r[-1]])
ri = np.array([0, 0.15])
l1, = axw.plot([0.5 * π, 0.5 * π], ro, color='k', lw=2, zorder=2)
l2, = axw.plot([0.5 * π, 0.5 * π], ri, color='k', lw=2, zorder=2)
l3, = axw.plot([-0.5 * π, -0.5 * π], ri, color='k', lw=2, zorder=2)
l4, = axw.plot([-0.5 * π, -0.5 * π], ro, color='k', lw=2, zorder=2)

axw.xaxis.set_major_formatter(plt.NullFormatter())
axw.yaxis.set_major_formatter(plt.NullFormatter())

fig.tight_layout()

axi = fig.add_subplot(122, polar='True')

lin, = axi.plot(x, farfield(0.5), color='C2')
axi.set_xlim(-0.5 * π, 0.5 * π)
axi.set_ylim(0., 4.01)
# axi.yaxis.set_major_locator(plt.MultipleLocator(0.1))

plt.subplots_adjust(right=0.90, left=-0.10, bottom=0.05, top=0.95)

dax = fig.add_axes([0.95, 0.10, 0.03, 0.8])

sst = {
    'track_color': '0.7',
    'color': '0.7',
    'initcolor': 'k',
}

d_slider = Slider(ax=dax, label='$d/\\lambda$', orientation='vertical', valmin=0.0, valmax=10, valinit=0.5, valstep=0.125, **sst)


def on_slider(val):
    global psi
    d = d_slider.val
    psi = wave(d)
    img.set_array(psi.real)
    lin.set_ydata(farfield(d))
    l1.set_ydata([0.5 * d + 0.15, r[-1]])
    l2.set_ydata([0., 0.5 * d - 0.15])
    l3.set_ydata([0., 0.5 * d - 0.15])
    l4.set_ydata([0.5 * d + 0.15, r[-1]])
    fig.canvas.draw_idle()


d_slider.on_changed(on_slider)


def animate(i):
    global psi
    psi *= δ
    img.set_array(psi.real)
    return img, l1, l2, l3, l4


anim = animation.FuncAnimation(fig, animate, interval=1, blit=True, cache_frame_data=False)

paused = False


def toggle_pause(ev):
    if ev.key != ' ': return
    global paused
    if paused:
        anim.resume()
    else:
        anim.pause()
    paused = not paused


fig.canvas.mpl_connect('key_press_event', toggle_pause)

plt.show()
